@@ -3,13 +3,13 @@
 // If the version of rust used is less than v1.63, please uncomment the follow attribute.
 // #![feature(explicit_generic_args_with_impl_trait)]
 
-use wasmedge_sdk::{params, Vm, WasmVal};
-use wasmedge_types::{wat2wasm, ValType};
+use wasmedge_sdk::{params, VmBuilder, WasmVal};
+use wasmedge_types::wat2wasm;
 
 #[cfg_attr(test, test)]
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     // create a Vm context
-    let vm = Vm::new(None)?;
+    let vm = VmBuilder::new().build()?;
 
     // register a wasm module from the given in-memory wasm bytes
     let wasm_bytes = wat2wasm(
@@ -45,20 +45,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
        )
     "#,
     )?;
-    let mut vm = vm.register_module_from_bytes("extern", wasm_bytes)?;
-
-    // get func type of `fib`
-    let func_ty = vm.func_ty(Some("extern"), "fib")?;
-
-    // get the argument types
-    assert_eq!(func_ty.args_len(), 1);
-    let args = func_ty.args().expect("No argument types.");
-    assert_eq!(args, [ValType::I32]);
-
-    // get the return types
-    assert_eq!(func_ty.returns_len(), 1);
-    let returns = func_ty.returns().expect("No return types.");
-    assert_eq!(returns, [ValType::I32]);
+    let vm = vm.register_module_from_bytes("extern", wasm_bytes)?;
 
     // run `fib` function in the named module instance
     let returns = vm.run_func(Some("extern"), "fib", params!(10))?;
