@@ -1,6 +1,8 @@
 //! Defines the structs used to construct configurations.
 
-use crate::{CompilerOptimizationLevel, CompilerOutputFormat, WasmEdgeResult};
+use crate::WasmEdgeResult;
+#[cfg(feature = "aot")]
+use crate::{CompilerOptimizationLevel, CompilerOutputFormat};
 use wasmedge_sys as sys;
 
 /// Defines a builder for creating a [Config].
@@ -8,6 +10,7 @@ use wasmedge_sys as sys;
 pub struct ConfigBuilder {
     common_config: CommonConfigOptions,
     stat_config: Option<StatisticsConfigOptions>,
+    #[cfg(feature = "aot")]
     compiler_config: Option<CompilerConfigOptions>,
     runtime_config: Option<RuntimeConfigOptions>,
     host_config: Option<HostRegistrationConfigOptions>,
@@ -18,6 +21,7 @@ impl ConfigBuilder {
         Self {
             common_config: options,
             stat_config: None,
+            #[cfg(feature = "aot")]
             compiler_config: None,
             runtime_config: None,
             host_config: None,
@@ -98,6 +102,7 @@ impl ConfigBuilder {
             inner.measure_cost(stat_config.measure_cost);
             inner.measure_time(stat_config.measure_time);
         }
+        #[cfg(feature = "aot")]
         if let Some(compiler_config) = self.compiler_config {
             inner.set_aot_compiler_output_format(compiler_config.out_format);
             inner.set_aot_optimization_level(compiler_config.opt_level);
@@ -110,20 +115,6 @@ impl ConfigBuilder {
         }
         if let Some(host_config) = self.host_config {
             inner.wasi(host_config.wasi);
-            #[cfg(target_os = "linux")]
-            inner.wasmedge_process(host_config.wasmedge_process);
-            #[cfg(all(target_os = "linux", feature = "wasi_nn", target_arch = "x86_64"))]
-            inner.wasi_nn(host_config.wasi_nn);
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            inner.wasi_crypto_common(host_config.wasi_crypto_common);
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            inner.wasi_crypto_asymmetric_common(host_config.wasi_crypto_asymmetric_common);
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            inner.wasi_crypto_symmetric(host_config.wasi_crypto_symmetric);
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            inner.wasi_crypto_kx(host_config.wasi_crypto_kx);
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            inner.wasi_crypto_signatures(host_config.wasi_crypto_signatures);
         }
 
         Ok(Config { inner })
@@ -137,7 +128,8 @@ impl ConfigBuilder {
 /// The following code shows how to create a [Config] with [ConfigBuilder].
 ///
 /// ```rust
-/// use wasmedge_sdk::{config::{Config, ConfigBuilder, CommonConfigOptions, StatisticsConfigOptions, CompilerConfigOptions, RuntimeConfigOptions, HostRegistrationConfigOptions}};
+///
+/// use wasmedge_sdk::{config::{Config, ConfigBuilder, CommonConfigOptions, StatisticsConfigOptions, RuntimeConfigOptions, HostRegistrationConfigOptions}};
 /// use wasmedge_types::{CompilerOutputFormat, CompilerOptimizationLevel};
 ///
 /// let common_options = CommonConfigOptions::default()
@@ -149,13 +141,6 @@ impl ConfigBuilder {
 ///     .sign_extension_operators(true)
 ///     .simd(true);
 ///
-/// let compiler_options = CompilerConfigOptions::default()
-///     .dump_ir(true)
-///     .generic_binary(true)
-///     .interruptible(true)
-///     .optimization_level(CompilerOptimizationLevel::O0)
-///     .out_format(CompilerOutputFormat::Native);
-///
 /// let stat_options = StatisticsConfigOptions::default()
 ///     .count_instructions(true)
 ///     .measure_cost(true)
@@ -168,7 +153,6 @@ impl ConfigBuilder {
 ///
 /// let result = ConfigBuilder::new(common_options)
 ///     .with_statistics_config(stat_options)
-///     .with_compiler_config(compiler_options)
 ///     .with_runtime_config(runtime_options)
 ///     .with_host_registration_config(host_options)
 ///     .build();
@@ -185,42 +169,6 @@ impl Config {
         self.inner.wasi_enabled()
     }
 
-    /// Checks if host registration wasmedge process turns on or not.
-    #[cfg(target_os = "linux")]
-    pub fn wasmedge_process_enabled(&self) -> bool {
-        self.inner.wasmedge_process_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_nn", target_arch = "x86_64"))]
-    pub fn wasi_nn_enabled(&self) -> bool {
-        self.inner.wasi_nn_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_common_enabled(&self) -> bool {
-        self.inner.wasi_crypto_common_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_asymmetric_common_enabled(&self) -> bool {
-        self.inner.wasi_crypto_asymmetric_common_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_symmetric_enabled(&self) -> bool {
-        self.inner.wasi_crypto_symmetric_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_kx_enabled(&self) -> bool {
-        self.inner.wasi_crypto_kx_enabled()
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_signatures_enabled(&self) -> bool {
-        self.inner.wasi_crypto_signatures_enabled()
-    }
-
     /// Returns the number of the memory pages available.
     pub fn max_memory_pages(&self) -> u32 {
         self.inner.get_max_memory_pages()
@@ -363,7 +311,7 @@ impl Config {
 ///  - `SIMD` supports 128-bit packed SIMD extension to WebAssembly.
 ///
 ///    Also see [SIMD Proposal](https://github.com/WebAssembly/spec/blob/main/proposals/simd/SIMD.md).
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub struct CommonConfigOptions {
     mutable_globals: bool,
     non_trap_conversions: bool,
@@ -380,6 +328,20 @@ pub struct CommonConfigOptions {
 }
 impl CommonConfigOptions {
     /// Creates a new instance of [CommonConfigOptions].
+    ///
+    /// The default options are:
+    /// * mutable_globals: true,
+    /// * non_trap_conversions: true,
+    /// * sign_extension_operators: true,
+    /// * multi_value: true,
+    /// * bulk_memory_operations: true,
+    /// * reference_types: true,
+    /// * simd: true,
+    /// * multi_memories: false,
+    /// * threads: false,
+    /// * tail_call: false,
+    /// * function_references: false,
+    /// * interpreter_mode: false,
     pub fn new() -> Self {
         Self {
             mutable_globals: true,
@@ -542,6 +504,21 @@ impl CommonConfigOptions {
     }
 }
 impl Default for CommonConfigOptions {
+    /// Creates a new default instance of [CommonConfigOptions].
+    ///
+    /// The default options are:
+    /// * mutable_globals: true,
+    /// * non_trap_conversions: true,
+    /// * sign_extension_operators: true,
+    /// * multi_value: true,
+    /// * bulk_memory_operations: true,
+    /// * reference_types: true,
+    /// * simd: true,
+    /// * multi_memories: false,
+    /// * threads: false,
+    /// * tail_call: false,
+    /// * function_references: false,
+    /// * interpreter_mode: false,
     fn default() -> Self {
         Self::new()
     }
@@ -570,8 +547,8 @@ impl Default for CommonConfigOptions {
 ///  - `interruptible` determines if AOT compiler generates interruptible binary or not.
 ///  
 ///  The configuration options above are only effective to [AOT compiler](crate::Compiler).
-#[derive(Debug)]
 #[cfg(feature = "aot")]
+#[derive(Debug, Clone, Copy)]
 pub struct CompilerConfigOptions {
     out_format: CompilerOutputFormat,
     opt_level: CompilerOptimizationLevel,
@@ -579,6 +556,7 @@ pub struct CompilerConfigOptions {
     generic_binary: bool,
     interruptible: bool,
 }
+#[cfg(feature = "aot")]
 impl CompilerConfigOptions {
     /// Creates a new instance of [CompilerConfigOptions].
     pub fn new() -> Self {
@@ -653,6 +631,7 @@ impl CompilerConfigOptions {
         }
     }
 }
+#[cfg(feature = "aot")]
 impl Default for CompilerConfigOptions {
     fn default() -> Self {
         Self::new()
@@ -665,7 +644,7 @@ impl Default for CompilerConfigOptions {
 ///
 /// - `maximum_memory_page` limits the page size of [Memory](crate::Memory). This option is only effective to
 ///       [Executor](crate::Executor).
-#[derive(Debug)]
+#[derive(Debug, Clone, Copy)]
 pub struct RuntimeConfigOptions {
     max_memory_pages: u32,
 }
@@ -703,7 +682,7 @@ impl Default for RuntimeConfigOptions {
 ///  - `measure_cost` determines if measuring the instruction costs when running a compiled or pure WASM.
 ///   
 ///  - `measure_time` determines if measuring the running time when running a compiled or pure WASM.
-#[derive(Debug, Default)]
+#[derive(Debug, Default, Clone, Copy)]
 pub struct StatisticsConfigOptions {
     count_instructions: bool,
     measure_cost: bool,
@@ -757,115 +736,28 @@ impl StatisticsConfigOptions {
 /// [HostRegistrationConfigOptions] is used to set the host registration configuration options, which are
 ///
 ///   - `Wasi` turns on the `WASI` support.
-///
-///   - `WasmEdgeProcess` turns on the `wasmedge_process` support.
-#[derive(Debug, Default)]
+#[derive(Debug, Default, Clone, Copy)]
 pub struct HostRegistrationConfigOptions {
     wasi: bool,
-    #[cfg(target_os = "linux")]
-    wasmedge_process: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_nn", target_arch = "x86_64"))]
-    wasi_nn: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    wasi_crypto_common: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    wasi_crypto_asymmetric_common: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    wasi_crypto_symmetric: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    wasi_crypto_kx: bool,
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    wasi_crypto_signatures: bool,
 }
 impl HostRegistrationConfigOptions {
+    /// Creates a new instance of [HostRegistrationConfigOptions].
+    pub fn new() -> Self {
+        Self::default()
+    }
+
     /// Enables or disables host registration wasi.
     ///
     /// # Argument
     ///
     /// - `enable` specifies if the option turns on or not.
     pub fn wasi(self, enable: bool) -> Self {
-        Self {
-            wasi: enable,
-            #[cfg(target_os = "linux")]
-            wasmedge_process: self.wasmedge_process,
-            #[cfg(all(target_os = "linux", feature = "wasi_nn", target_arch = "x86_64"))]
-            wasi_nn: self.wasi_nn,
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            wasi_crypto_common: self.wasi_crypto_common,
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            wasi_crypto_asymmetric_common: self.wasi_crypto_asymmetric_common,
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            wasi_crypto_symmetric: self.wasi_crypto_symmetric,
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            wasi_crypto_kx: self.wasi_crypto_kx,
-            #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-            wasi_crypto_signatures: self.wasi_crypto_signatures,
-        }
-    }
-
-    /// Enables or disables host registration WasmEdge process.
-    ///
-    /// # Argument
-    ///
-    /// - `enable` specifies if the option turns on or not.
-    #[cfg(target_os = "linux")]
-    pub fn wasmedge_process(self, enable: bool) -> Self {
-        Self {
-            wasmedge_process: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_nn", target_arch = "x86_64"))]
-    pub fn wasi_nn(self, enable: bool) -> Self {
-        Self {
-            wasi_nn: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_common(self, enable: bool) -> Self {
-        Self {
-            wasi_crypto_common: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_asymmetric_common(self, enable: bool) -> Self {
-        Self {
-            wasi_crypto_asymmetric_common: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_symmetric(self, enable: bool) -> Self {
-        Self {
-            wasi_crypto_symmetric: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_kx(self, enable: bool) -> Self {
-        Self {
-            wasi_crypto_kx: enable,
-            ..self
-        }
-    }
-
-    #[cfg(all(target_os = "linux", feature = "wasi_crypto"))]
-    pub fn wasi_crypto_signatures(self, enable: bool) -> Self {
-        Self {
-            wasi_crypto_signatures: enable,
-            ..self
-        }
+        Self { wasi: enable }
     }
 }
 
 #[cfg(test)]
+#[cfg(feature = "aot")]
 mod tests {
     use super::*;
 
