@@ -92,7 +92,6 @@ class Executor {
 public:
   Executor(const Configure &Conf, Statistics::Statistics *S = nullptr) noexcept
       : Conf(Conf) {
-    assuming(This == nullptr);
     if (Conf.getStatisticsConfigure().isInstructionCounting() ||
         Conf.getStatisticsConfigure().isCostMeasuring() ||
         Conf.getStatisticsConfigure().isTimeMeasuring()) {
@@ -100,13 +99,11 @@ public:
     } else {
       Stat = nullptr;
     }
-    newThread();
     if (Stat) {
       Stat->setCostLimit(Conf.getStatisticsConfigure().getCostLimit());
     }
   }
   ~Executor() noexcept {
-    This = nullptr;
     ExecutionContext.StopToken = nullptr;
     ExecutionContext.InstrCount = nullptr;
     ExecutionContext.CostTable = nullptr;
@@ -131,18 +128,6 @@ public:
   invoke(const Runtime::Instance::FunctionInstance &FuncInst,
          Span<const ValVariant> Params, Span<const ValType> ParamTypes);
 
-  /// Register new thread
-  void newThread() noexcept {
-    This = this;
-    ExecutionContext.StopToken = &StopToken;
-    if (Stat) {
-      ExecutionContext.InstrCount = &Stat->getInstrCountRef();
-      ExecutionContext.CostTable = Stat->getCostTable().data();
-      ExecutionContext.Gas = &Stat->getTotalCostRef();
-      ExecutionContext.GasLimit = Stat->getCostLimit();
-    }
-  }
-
   /// Stop execution
   void stop() noexcept {
     StopToken.store(1, std::memory_order_relaxed);
@@ -637,12 +622,12 @@ public:
   static const AST::Module::IntrinsicsTable Intrinsics;
 
 private:
-  Expect<uint32_t> atomicNotify(Runtime::Instance::MemoryInstance &MemInst,
-                                uint32_t Address, uint32_t Count) noexcept;
   template <typename T>
   Expect<uint32_t> atomicWait(Runtime::Instance::MemoryInstance &MemInst,
                               uint32_t Address, T Expected,
                               int64_t Timeout) noexcept;
+  Expect<uint32_t> atomicNotify(Runtime::Instance::MemoryInstance &MemInst,
+                                uint32_t Address, uint32_t Count) noexcept;
   void atomicNotifyAll() noexcept;
 
   struct Waiter {
@@ -655,6 +640,22 @@ private:
   std::unordered_multimap<uint32_t, Waiter> WaiterMap;
 
 private:
+  /// Prepare execution context
+  void prepare(Runtime::StackManager &StackMgr, uint8_t *const *Memories,
+               ValVariant *const *Globals) noexcept {
+    This = this;
+    ExecutionContext.StopToken = &StopToken;
+    ExecutionContext.Memories = Memories;
+    ExecutionContext.Globals = Globals;
+    if (Stat) {
+      ExecutionContext.InstrCount = &Stat->getInstrCountRef();
+      ExecutionContext.CostTable = Stat->getCostTable().data();
+      ExecutionContext.Gas = &Stat->getTotalCostRef();
+      ExecutionContext.GasLimit = Stat->getCostLimit();
+    }
+    CurrentStack = &StackMgr;
+  }
+
   /// Execution context for compiled functions
   struct ExecutionContextStruct {
     uint8_t *const *Memories;
