@@ -68,10 +68,14 @@ typedef struct BOOST_MAY_ALIAS _IO_STATUS_BLOCK {
 } IO_STATUS_BLOCK_, *PIO_STATUS_BLOCK_;
 
 #if defined(BOOST_USE_WINDOWS_H)
+BOOST_CONSTEXPR_OR_CONST DWORD_ FILE_APPEND_DATA_ = FILE_APPEND_DATA;
+BOOST_CONSTEXPR_OR_CONST DWORD_ FILE_TYPE_CHAR_ = FILE_TYPE_CHAR;
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_INPUT_HANDLE_ = STD_INPUT_HANDLE;
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_OUTPUT_HANDLE_ = STD_OUTPUT_HANDLE;
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_ERROR_HANDLE_ = STD_ERROR_HANDLE;
 #else
+BOOST_CONSTEXPR_OR_CONST DWORD_ FILE_APPEND_DATA_ = static_cast<DWORD_>(0x4);
+BOOST_CONSTEXPR_OR_CONST DWORD_ FILE_TYPE_CHAR_ = static_cast<DWORD_>(0x2);
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_INPUT_HANDLE_ = static_cast<DWORD_>(-10);
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_OUTPUT_HANDLE_ = static_cast<DWORD_>(-11);
 BOOST_CONSTEXPR_OR_CONST DWORD_ STD_ERROR_HANDLE_ = static_cast<DWORD_>(-12);
@@ -669,8 +673,8 @@ inline constexpr int toSockType(__wasi_sock_type_t SockType) noexcept {
   }
 }
 
-inline constexpr __wasi_protocol_t fromProtocal(int Protocal) noexcept {
-  switch (Protocal) {
+inline constexpr __wasi_protocol_t fromProtocol(int Protocol) noexcept {
+  switch (Protocol) {
   case IPPROTO_IP:
     return __WASI_PROTOCOL_IPPROTO_IP;
   case IPPROTO_TCP:
@@ -682,8 +686,8 @@ inline constexpr __wasi_protocol_t fromProtocal(int Protocal) noexcept {
   }
 }
 
-inline constexpr int toProtocal(__wasi_protocol_t Protocal) noexcept {
-  switch (Protocal) {
+inline constexpr int toProtocol(__wasi_protocol_t Protocol) noexcept {
+  switch (Protocol) {
   case __WASI_PROTOCOL_IPPROTO_IP:
     return IPPROTO_IP;
   case __WASI_PROTOCOL_IPPROTO_TCP:
