@@ -7,7 +7,7 @@
 int main(int Argc, const char *Argv[]) {
 #if WASMEDGE_WINCLI_HELPER_ENABLE
   WasmEdge_SetConsoleOutputCPtoUTF8();
-  (void)Argv; // We can not ensure the encoding of Argv, make compiler slient.
+  (void)Argv; // We can not ensure the encoding of Argv, make compiler silent.
   auto ArgvUTF8 =
       WasmEdge_CommandLineToUTF8ArgvW(WasmEdge_GetCommandLineW(), &Argc);
   return WasmEdge_Driver_Compiler(Argc, ArgvUTF8);
